@@ -4,46 +4,16 @@
 convert a MAVLink tlog file to a MATLab mfile
 '''
 
-import sys
+import sys, os
 import re
 from pymavlink import mavutil
 
-def create_mfile(filename, msg_types, msg_lists):
-    '''create a .m file from msg_types and msg_lists'''
-    f = open(filename, mode='w')
-
-    # put out all the headings
-    for mtype in msg_types:
-        m = msg_types[mtype]
-        fieldnames = m._fieldnames
-        f.write("%s.heading = {'timestamp'" % mtype)
-        for field in fieldnames:
-            val = getattr(m, field)
-            if not isinstance(val, str):
-                f.write(",'%s'" % field)
-        f.write("};\n")
-    for mtype in msg_lists:
-        f.write("%s.data = [" % mtype)
-        for m in msg_lists[mtype]:
-            fieldnames = m._fieldnames
-
-            # always include the timestamp
-            f.write("%f" % m._timestamp)
-            for field in fieldnames:
-                val = getattr(m, field)
-                if not isinstance(val, str):
-                    f.write(",%f" % val)
-            f.write(";\n")
-        f.write("];\n\n")
-
-    f.close()
-
 def process_tlog(filename):
     '''convert a tlog to a .m file'''
 
     print("Processing %s" % filename)
     
-    mlog = mavutil.mavlink_connection(filename, dialect=opts.dialect)
+    mlog = mavutil.mavlink_connection(filename, dialect=opts.dialect, zero_time_base=True)
     
     # first walk the entire file, grabbing all messages into a hash of lists,
     #and the first message of each type into a hash
@@ -54,6 +24,18 @@ def process_tlog(filename):
     if types is not None:
         types = types.split(',')
 
+    # note that Octave doesn't like any extra '.', '*', '-', characters in the filename
+    (head, tail) = os.path.split(filename)
+    basename = '.'.join(tail.split('.')[:-1])
+    mfilename = re.sub('[\.\-\+\*]','_', basename) + '.m'
+    if head is not None:
+        mfilename = os.path.join(head, mfilename)
+    print("Creating %s" % mfilename)
+
+    f = open(mfilename, "w")
+
+    type_counters = {}
+
     while True:
         m = mlog.recv_match(condition=opts.condition)
         if m is None:
@@ -64,17 +46,28 @@ def process_tlog(filename):
         if m.get_type() == 'BAD_DATA':
             continue
         
-        if m.get_type() not in msg_lists.keys():
-            msg_lists[m.get_type()] = []
-            msg_types[m.get_type()] = m
+        fieldnames = m._fieldnames
+        mtype = m.get_type()
+        if mtype in ['FMT', 'PARM']:
+            continue
 
-        msg_lists[m.get_type()].append(m)
+        if mtype not in type_counters:
+            type_counters[mtype] = 0
+            f.write("%s.heading = {'timestamp'" % mtype)
+            for field in fieldnames:
+                val = getattr(m, field)
+                if not isinstance(val, str):
+                    f.write(",'%s'" % field)
+            f.write("};\n")
 
-    # note that Octave doesn't like any extra '.', '*', '-', characters in the filename
-    basename = '.'.join(filename.split('.')[:-1])
-    mfilename = re.sub('[\.\-\+\*]','_', basename) + '.m'
-    print("Creating %s" % mfilename)
-    create_mfile(mfilename, msg_types, msg_lists)
+        type_counters[mtype] += 1
+        f.write("%s.data(%u,:) = [%f" % (mtype, type_counters[mtype], m._timestamp))
+        for field in m._fieldnames:
+            val = getattr(m, field)
+            if not isinstance(val, str):
+                f.write(",%f" % val)
+        f.write("];\n")
+    f.close()
 
 from optparse import OptionParser
 parser = OptionParser("mavtomfile.py [options]")
